
Object subclass: #PuzzleEntry
  instanceVariableNames: 
    ' where string restoration order scaffold '
  classVariableNames: ''
  poolDictionaries: 
    ' ColorConstants '!

PuzzleEntry subclass: #PuzzleAcrossEntry
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: 
    ' ColorConstants '!

PuzzleEntry subclass: #PuzzleDownEntry
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: 
    ' ColorConstants '!

Object subclass: #PuzzleScaffold
  instanceVariableNames: 
    ' entries extent grid strategy first next smallFont largeFont squareSize pleaseStop statusWidget '
  classVariableNames: ''
  poolDictionaries: 
    ' ColorConstants '!

ViewManager subclass: #PuzzleMaker
  instanceVariableNames: 
    ' changed puzzleScaffold '
  classVariableNames: ''
  poolDictionaries: 
    ' ColorConstants OperatingSystemConstants '!



!PuzzleEntry methods "for 'get/set'" !
 
string: aString
    string := aString!
 
order
    ^order!
  
order: anInteger
    order := anInteger!
   
scaffold
    ^scaffold!

scaffold: aPuzzleScaffold
    scaffold := aPuzzleScaffold!
 
where: aPoint
    where := aPoint!
 
where
    ^where!
  
string
    ^string! !

!PuzzleEntry methods "for 'scaffold queries'" !
   
width
    ^self scaffold width!

largeFontOffset
    ^self scaffold largeFontOffset!

smallFont
    ^self scaffold smallFont!

smallFontOffset
    ^self scaffold smallFontOffset!

largeFont
    ^self scaffold largeFont!

squareOffsetFor: aCoordinate
    ^self scaffold squareOffsetFor: aCoordinate!
  
extent
    ^self scaffold extent!
  
next
    ^self scaffold nextAfter: self where!
 
height
    ^self scaffold height!
  
squareSize
    ^self scaffold squareSize! !

!PuzzleEntry methods "for 'gridding'" !
 
stringCanOverlap: gridEntries
    gridEntries with: self string do: [:gridEntry :character |
        ((gridEntry == character) or: [gridEntry == $ ])
            ifFalse: [^false]].
    ^true
!
  
gridEntries: entries
    "Sets the grid entries for this puzzle entry; either characters or nil."
    1 to: entries size do: [:index |
        self gridAt: index put: (entries at: index)].!

gridEntries
    "Returns the grid entries for this puzzle entry; either characters or nil."
    | entries |
    entries := Array new: self string size.
    1 to: entries size do: [:index |
        entries at: index  put: (self gridAt: index)].
    ^entries!
 
grid
    ^self scaffold grid.! !

!PuzzleEntry methods "for 'success/failure'" !
 
fail
    self where: nil!
  
succeeded
    ^self failed not!

failed
    ^self where isNil! !

!PuzzleEntry methods "for 'controlling'" !
  
continue
    [true] whileTrue: [
        self where: self next.
        self failed ifTrue: [^self].
        self legitimate ifTrue: [
            self add ifTrue: [^self]]].
!
  
reset
    self where: self scaffold first.
    self add ifTrue: [^self].
    self continue
!

set
    self failed ifTrue: [^self].
    self remove.
    self continue
! !

!PuzzleEntry methods "for 'arranging'" !
 
add
    "Returns if succeeded or not."
    (self stringCanFit) ifFalse: [^false].
    restoration := self gridEntries.
    (self stringCanOverlap: restoration) ifFalse: [^false].
    self gridEntries: self string.
    ^true!
  
remove
    self gridEntries: restoration! !

!PuzzleEntry methods "for 'drawing'" !
  
drawUsingPen: pen
    | start |
    pen font: self largeFont.
    self coordinatesAndCharacterDo: [:aCoordinate :character |
            start := self squareOffsetFor: aCoordinate.
            "Make white background."
            pen
                fill: (start + 1 corner: start + self squareSize - 1)
                color: ClrWhite.
            "Draw large character."
            pen
                place: start + self largeFontOffset;
                centerText: (String with: character)].

    "Draw small entry order."
    pen font: self smallFont.
    start := self squareOffsetFor: self firstCoordinate.
    pen
        place: start + self smallFontOffset;
        displayText: self order printString.! !



!PuzzleAcrossEntry methods "for 'gridding'" !
 
stringCanFit
    ^(self where x + self string size - 1) <= (self width + 1)!
   
legitimate
    "Across entries can't be at y=0 or y=height+1."
    ^self where y between: 1 and: self height!
 
gridAt: index put: anObject
    self grid at: self where + ((index - 1)@0) put: anObject!
  
gridAt: index
    ^self grid at: self where + ((index - 1)@0)! !

!PuzzleAcrossEntry methods "for 'printing'" !
  
printOn: aStream
    aStream nextPutAll: self string; nextPutAll: 'across'.! !

!PuzzleAcrossEntry methods "for 'drawing'" !
 
drawUsingPen: pen
    pen foreColor: ClrRed.
    super drawUsingPen: pen!
 
coordinatesAndCharacterDo: aBlock
    "Provide coordinates and characters
    which are not blank."
    2 to: self string size - 1 do: [:index |
        aBlock
            value: self where + ((index - 1)@0)
            value: (self string at: index)]!
  
firstCoordinate
    "Provide the coordinates for the first non-blank characters."
    ^self where + (1@0)! !



!PuzzleDownEntry methods "for 'gridding'" !
   
stringCanFit
    ^(self where y + self string size - 1) <= (self height + 1)!
  
legitimate
    "Across entries can't be at x=0 or x=width+1."
    ^self where x between: 1 and: self width!
   
gridAt: index put: anObject
    self grid at: self where + (0@(index - 1)) put: anObject!
  
gridAt: index
    ^self grid at: self where + (0@(index - 1))! !

!PuzzleDownEntry methods "for 'printing'" !

printOn: aStream
    aStream nextPutAll: self string; nextPutAll: 'down'.! !

!PuzzleDownEntry methods "for 'drawing'" !
 
drawUsingPen: pen
    pen foreColor: ClrBlue.
    super drawUsingPen: pen!

coordinatesAndCharacterDo: aBlock
    "Provide coordinates and characters
    which are not blank."
    2 to: self string size - 1 do: [:index |
        aBlock
            value: self where + (0@(index - 1))
            value: (self string at: index)]!
  
firstCoordinate
    "Provide the coordinates for the first non-blank characters."
    ^self where + (0@1)! !


!PuzzleScaffold class methods "for 'comment'" !
 
comment
    "The grid is constructed to have an extra border of
    spaces all around to take into account the fact that
    each puzzle entry is padded with a space at both
    ends."! !

!PuzzleScaffold class methods "for 'instance creation'" !

new
    ^super new initialize! !


!PuzzleScaffold methods "for 'initializing'" !
   
initializeFonts
    self smallFont: (Font
        face: 'Times New Roman'
        size: 8
        fixedWidth: false).
    self largeFont: (Font
        face: 'System 10 bold'
        size: 10
        fixedWidth: false)!
 
initialize
    self initializeFonts.
    self pleaseStop: false!
  
reinitializeGrid
    "Create a grid with an extra border all around."
    grid := Dictionary new.
    0 to: self extent y + 1 do: [:y |
        0 to: self extent x + 1 do: [:x |
            grid at: x@y put: $ ]]! !

!PuzzleScaffold methods "for 'get/set'" !
   
statusWidget: aWidget
    statusWidget := aWidget!
 
width
    ^self extent x!
  
extent: aPoint
    extent := aPoint.
    self reinitializeGrid
!
 
extent
    ^extent!

statusWidget
    ^statusWidget!

pleaseStop
    ^pleaseStop!

grid: aDictionary
    grid := aDictionary!
 
pleaseStop: aBoolean
    pleaseStop := aBoolean!
   
height
    ^self extent y!
 
grid
    ^grid! !

!PuzzleScaffold methods "for 'entries'" !
 
entries: anOrderedCollection
    entries := anOrderedCollection!
   
squareSize
    ^squareSize!

squareOffsetFor: aCoordinate
    ^(aCoordinate - 1) * squareSize!
  
entries
    ^entries!
  
acrossStrings: acrossStrings downStrings: downStrings
    "Create the entries, alternate across and down."
    | acrossEntries downEntries |
    acrossEntries := acrossStrings collect: [:string |
        PuzzleAcrossEntry new
            string: ' ', string, ' ';
            scaffold: self].
    downEntries := downStrings collect: [:string |
            PuzzleDownEntry new
                string: ' ', string, ' ';
                scaffold: self].
    entries := OrderedCollection new.
    [(acrossEntries size + downEntries size) > 0] whileTrue: [
        acrossEntries notEmpty ifTrue: [
            entries add: acrossEntries removeFirst].
        downEntries notEmpty ifTrue: [
            entries add: downEntries removeFirst]].
! !

!PuzzleScaffold methods "for 'ordering'" !
 
nextAfter: previous
    ^next at: previous!

first
    ^first!
  
link: previous to: after
    "Ignore attempts to link to itself, to out of bounds points,
    or to a prior element in the list."
    (previous = after or: [
    after notNil and: [
    (after x between: 0 and: self extent x + 1) not or: [
    (after y between: 0 and: self extent y + 1) not]]])
        ifTrue: [^previous].
    (next includesKey: after)
        ifTrue: [^previous].
    next at: previous put: after.
    ^after
! !

!PuzzleScaffold methods "for 'ordering strategy'" !
 
strategy
    ^strategy!

strategy: aSymbol
    "One of the symbols in strategies."
    strategy := aSymbol.
    self perform: self strategy!
  
strategies
    ^#(centerOut topDown random)! !

!PuzzleScaffold methods "for 'ordering support'" !
   
topDown
    | center previous after maximum minimum |
    next := Dictionary new.
    previous := first := 0@0.
    0 to: self extent y + 1 do: [:y |
        0 to: self extent x + 1 do: [:x |
            previous := self link: previous to: x@y]].
    self link: previous to: nil!
  
centerOut
    | center previous after maximum minimum |
    next := Dictionary new.
    center := self extent // 2. previous := first := center.
    0 to: (center x max: center y) + 1 do: [:radius |
        minimum := center - radius. maximum := center + radius.
        "Top left to right."
        minimum x to: maximum x do: [:x |
            previous := self link: previous to: x@minimum y].
        "Right top to bottom."
        minimum y to: maximum y do: [:y |
            previous := self link: previous to: maximum x@y].
        "Bottom right to left."
        maximum x to: minimum x by: -1 do: [:x |
            previous := self link: previous to: x@maximum y].
        "Left bottom to top."
        maximum y to: minimum y by: - 1 do: [:y |
            previous := self link: previous to: minimum x@y]].
    self link: previous to: nil!
 
random
    | keys randomizedKeys index previous |
    "Make up the keys."
    keys := OrderedCollection new.
    0 to: self extent y + 1 do: [:y |
        0 to: self extent x + 1 do: [:x |
            keys add: x@y]].

    "Choose them in a random order."
    randomizedKeys := OrderedCollection new.
    [keys notEmpty] whileTrue: [
        index := Time randomFrom1To: keys size.
        randomizedKeys add: (keys at: index).
        keys removeIndex: index].

    "Link the randomized keys together."
    next := Dictionary new.
    previous := first := randomizedKeys removeFirst.
    [randomizedKeys notEmpty] whileTrue: [
            previous := self link: previous to: randomizedKeys removeFirst].
    self link: previous to: nil! !

!PuzzleScaffold methods "for 'success/failure'" !

succeeded
    ^entries last succeeded!
 
failed
    ^entries last failed! !

!PuzzleScaffold methods "for 'controlling'" !

reset
    "Compute first legitimate values."
    self reinitializeGrid.
    self resetEntriesFrom: 1.
    self lastSuccessfulEntry = entries size
        ifTrue: [^self].
    self set
!

set
    | start entry misfits trials oldMisfits choices |
    start := self lastSuccessfulEntry.

    choices := self width * self height.
    misfits := entries size - start.
    trials := 1.
    misfits timesRepeat: [trials := trials * choices].

    [true] whileTrue: [
        "Notifier yield."
        self pleaseStop ifTrue: [self pleaseStop: false. ^self].
        start = 0 ifTrue: [^self].
        entry := entries at: start.
        entry set.
        entry succeeded
            ifTrue: [self resetEntriesFrom: start + 1]
            ifFalse: [
                oldMisfits := misfits.
                misfits := (entries size - start + 1) max: misfits.
                misfits > oldMisfits ifTrue: [
                    trials := 1.
                    misfits timesRepeat: [trials := trials * choices]]].
        statusWidget contents: 'Words that don''t fit: ',
            misfits printString, ', choices to try: ', trials printString.
        trials := trials - 1.
        start := self lastSuccessfulEntry.
        start = entries size ifTrue: [^self]]! !

!PuzzleScaffold methods "for 'controlling support'" !
  
resetEntriesFrom: index1
    "Reset all entries. Fail all subsequent entries upon first failure. Return failure."
    | failed entry |
    failed := false.
    index1 to: entries size do: [:index |
        entry := entries at: index.
        failed
            ifTrue: [entry fail]
            ifFalse: [
                entry reset.
                failed := entry failed]].
    ^failed!
 
lastSuccessfulEntry
    entries size to: 1 by: -1 do: [:index |
        (entries at: index) succeeded
            ifTrue: [^index]].
    ^0! !

!PuzzleScaffold methods "for 'fonts'" !
   
smallFont: aFont
    smallFont := aFont!
   
smallFont
    ^smallFont!
  
largeFontOffset
    ^(self squareSize // 2) + (0@3).!
  
smallFontOffset
    ^smallFont charSize - 1!
   
largeFont: aFont
    largeFont := aFont!
   
largeFont
    ^largeFont! !

!PuzzleScaffold methods "for 'drawing'" !
   
sort: unsortedEntries
    | sortedEntries |
    sortedEntries := unsortedEntries
        asSortedCollection: [:entry1 :entry2 |
            entry1 where < entry2 where or: [
            entry1 where y < entry2 where y or: [
            entry1 where y = entry2 where y and: [
            entry1 where x < entry2 where x]]]].
    sortedEntries with: (1 to: sortedEntries size)
        do: [:entry :index |
            entry order: index]!
  
drawOn: aWidget
    | pen |
    entries isNil ifTrue: [^self].
    squareSize := aWidget extent // self extent.
    self succeeded
        ifTrue: [
            self sortEntries.
            pen := aWidget pen.
            pen fill: ClrBlack.
            entries do: [:entry |
                entry drawUsingPen: pen]]
        ifFalse: [aWidget pen fill: ClrRed]
!

sortEntries
    self sort: (entries select: [:entry |
        entry isKindOf: PuzzleAcrossEntry]).
    self sort: (entries select: [:entry |
        entry isKindOf: PuzzleDownEntry])! !


!PuzzleMaker class methods "for 'window builder'" !
  
wbCreated

    ^true! !

!PuzzleMaker class methods "for 'examples'" !
  
example1
    "PuzzleMaker example1"
    PuzzleMaker new open! !


!PuzzleMaker methods "for 'initializing'" !
  
initialize
    super initialize.
    changed := true.
    puzzleScaffold := PuzzleScaffold new extent: 15@15; strategy: #random.! !

!PuzzleMaker methods "for 'window builder'" !
 
createMenus: aPane

	"WARNING!! This method was automatically generated by 
	WindowBuilder. Code you add here which does not conform to 
	the WindowBuilder API will probably be lost the next time 
	you save your layout definition."

	#generated.
	aPane menuWindow
	    yourself;
	    addMenu: (
	        Menu new
	            title: 'File';
	            owner: self;
	            appendItem: 'Open...' selector: #fileOpen acceleratorString: '';
	            appendItem: 'Save...' selector: #fileSave acceleratorString: ''
	    )!
   
createViews

	"WARNING!! This method was automatically generated by 
	WindowBuilder. Code you add here which does not conform to 
	the WindowBuilder API will probably be lost the next time 
	you save your layout definition."

	| acrossListWidget button1 button2 button3 button4 button5 
	button6 downListWidget drawnButton1 drawnButton2 
	entryWidget mainView puzzleWidget statusWidget xDU yDU |
	
	#generated.
	xDU := (SysFont width + 1 roundTo: 2) / 4.
	yDU := SysFont height / 8.
	mainView := self topPaneClass new.
	
		"Temporary Variables"
	acrossListWidget := WBComboBox new.
	button1 := Button new.
	button2 := Button new.
	button3 := Button new.
	button4 := Button new.
	button5 := Button new.
	button6 := Button new.
	downListWidget := WBComboBox new.
	drawnButton1 := DrawnButton new.
	drawnButton2 := DrawnButton new.
	entryWidget := EnhancedEntryField new.
	puzzleWidget := GraphPane new.
	statusWidget := StaticText new.
	
	mainView
		owner: self;
		setName: 'mainView';
		when: #activated send: #activated to: self;
		labelWithoutPrefix: 'Puzzle Maker';
		noSmalltalkMenuBar;
		backColor: Color buttonFace;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1/2; leftInset: -131 * xDU;
				topRatio: 1/2; topInset: -134 * yDU;
				rightRatio: 1/2; rightInset: -131 * xDU;
				bottomRatio: 1/2; bottomInset: -134 * yDU).
	self addView: mainView.
	
	entryWidget "EnhancedEntryField"
		owner: self;
		setName: 'entryWidget';
		framingBlock: 
			(LayoutFrame new
				leftRatio: 45/128; leftInset: 0 * xDU;
				topRatio: 4/221; topInset: 0 * yDU;
				rightRatio: 83/128; rightInset: 0 * xDU;
				bottomRatio: 16/221; bottomInset: 0 * yDU);
		startGroup;
		font: SysFont.
	mainView
		addSubpane: entryWidget;
		subPaneWithFocus: entryWidget.
	
	drawnButton1
		owner: self;
		when: #clicked send: #clickedMoveRight to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 57/128; leftInset: 0 * xDU;
				topRatio: 22/221; topInset: 0 * yDU;
				rightRatio: 61/128; rightInset: 0 * xDU;
				bottomRatio: 30/221; bottomInset: 0 * yDU);
		cpContents: 'C:\STEXPRES\RARROW.BMP';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton1.
	
	drawnButton2
		owner: self;
		when: #clicked send: #clickedMoveLeft to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 67/128; leftInset: 0 * xDU;
				topRatio: 22/221; topInset: 0 * yDU;
				rightRatio: 71/128; rightInset: 0 * xDU;
				bottomRatio: 30/221; bottomInset: 0 * yDU);
		cpContents: 'C:\STEXPRES\LARROW.BMP';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton2.
	
	button1
		owner: self;
		when: #clicked send: #clickedAddLeft to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 19/129; leftInset: 0 * xDU;
				topRatio: 8/491; topInset: 0 * yDU;
				rightRatio: 44/129; rightInset: 0 * xDU;
				bottomRatio: 32/491; bottomInset: 0 * yDU);
		contents: 'Add Across';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: button1.
	
	button2
		owner: self;
		when: #clicked send: #clickedAddRight to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 85/129; leftInset: 0 * xDU;
				topRatio: 8/491; topInset: 0 * yDU;
				rightRatio: 110/129; rightInset: 0 * xDU;
				bottomRatio: 32/491; bottomInset: 0 * yDU);
		contents: 'Add Down';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: button2.
	
	button3
		owner: self;
		when: #clicked send: #clickedDeleteLeft to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1/43; leftInset: 0 * xDU;
				topRatio: 44/491; topInset: 0 * yDU;
				rightRatio: 73/516; rightInset: 0 * xDU;
				bottomRatio: 68/491; bottomInset: 0 * yDU);
		contents: 'Delete';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: button3.
	
	button4
		owner: self;
		when: #clicked send: #clickedDeleteRight to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 443/516; leftInset: 0 * xDU;
				topRatio: 44/491; topInset: 0 * yDU;
				rightRatio: 42/43; rightInset: 0 * xDU;
				bottomRatio: 68/491; bottomInset: 0 * yDU);
		contents: 'Delete';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: button4.
	
	puzzleWidget "GraphPane"
		owner: self;
		setName: 'puzzleWidget';
		when: #display send: #updateGraphics to: self;
		when: #needsContents send: #updateGraphics to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1/43; leftInset: 0 * xDU;
				topRatio: 112/491; topInset: 0 * yDU;
				rightRatio: 125/129; rightInset: 0 * xDU;
				bottomRatio: 476/491; bottomInset: 0 * yDU);
		removeHorizontalScrollbarStyle;
		removeVerticalScrollbarStyle;
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: puzzleWidget.
	
	button5
		owner: self;
		when: #clicked send: #clickedMake to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 37/43; leftInset: 0 * xDU;
				topRatio: 8/491; topInset: 0 * yDU;
				rightRatio: 505/516; rightInset: 0 * xDU;
				bottomRatio: 32/491; bottomInset: 0 * yDU);
		contents: 'Make';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: button5.
	
	button6
		owner: self;
		when: #clicked send: #clickedStop to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1/43; leftInset: 0 * xDU;
				topRatio: 8/491; topInset: 0 * yDU;
				rightRatio: 6/43; rightInset: 0 * xDU;
				bottomRatio: 32/491; bottomInset: 0 * yDU);
		contents: 'Stop';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: button6.
	
	acrossListWidget "WBComboBox"
		owner: self;
		setName: 'acrossListWidget';
		framingBlock: 
			(LayoutFrame new
				leftRatio: 19/128; leftInset: 0 * xDU;
				topRatio: 20/221; topInset: 0 * yDU;
				rightRatio: 55/128; rightInset: 0 * xDU;
				bottomRatio: 36/221; bottomInset: 0 * yDU);
		dropDownList;
		addDynamicListSizeStyle;
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: acrossListWidget.
	
	downListWidget "WBComboBox"
		owner: self;
		setName: 'downListWidget';
		framingBlock: 
			(LayoutFrame new
				leftRatio: 73/128; leftInset: 0 * xDU;
				topRatio: 20/221; topInset: 0 * yDU;
				rightRatio: 109/128; rightInset: 0 * xDU;
				bottomRatio: 36/221; bottomInset: 0 * yDU);
		dropDownList;
		addDynamicListSizeStyle;
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: downListWidget.
	
	statusWidget "StaticText"
		owner: self;
		setName: 'statusWidget';
		framingBlock: 
			(LayoutFrame new
				leftRatio: 0; leftInset: 48 * xDU;
				topRatio: 0; topInset: 42 * yDU;
				rightRatio: 0; rightInset: -252 * xDU;
				bottomRatio: 0; bottomInset: -50 * yDU);
		contents: 'Words that don''t fit: 0, choices to try: 10000000';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: statusWidget.
	
	self createMenus: mainView! !

!PuzzleMaker methods "for 'activation event handlers'" !
 
activated
    self updateGraphics! !

!PuzzleMaker methods "for 'file event handlers'" !
 
fileSave
    | fileName |
    fileName := (FileDialog new saveFile: '*.puz') file.
    fileName isNil ifTrue: [^self "user cancelled"].
    ObjectFiler
        dump: (Array with: self grid with: self fileSaveEntries)
        newFile: fileName!
   
fileOpen
    | fileName gridAndEntries grid entries |
    fileName := (FileDialog new
        title: 'Open File';
        fileSpec: '*.puz';
        hideReadonly;
        open) file.
    fileName isNil ifTrue: [^self "user cancelled"].
    gridAndEntries := ObjectFiler loadFromPathName: fileName.
    grid := gridAndEntries first. entries := gridAndEntries last.
    entries do: [:entry | entry scaffold: puzzleScaffold].
    puzzleScaffold grid: grid; entries: entries.
    self acrossListWidget contents: self acrossStringsFromEntries.
    self downListWidget contents: self downStringsFromEntries.
    self updateGraphics
! !

!PuzzleMaker methods "for 'file event handler support'" !
   
grid
    ^puzzleScaffold grid!
 
downEntries
    ^puzzleScaffold entries select: [:entry |
        entry isKindOf: PuzzleDownEntry]!
   
downStringsFromEntries
    ^self stringsFromEntries: self downEntries!
 
acrossEntries
    ^puzzleScaffold entries select: [:entry |
        entry isKindOf: PuzzleAcrossEntry]!
   
fileSaveEntries
    ^puzzleScaffold entries collect: [:entry |
        entry shallowCopy scaffold: nil; yourself]!

stringsFromEntries: entries
    ^entries collect: [:entry |
        entry string copyFrom: 2 to: entry string size - 1]
!

acrossStringsFromEntries
    ^self stringsFromEntries: self acrossEntries! !

!PuzzleMaker methods "for 'clicked event handling'" !
  
clickedAddRight
    self add: (self entryWidget contents) toWidget: self downListWidget!
   
clickedDeleteLeft
    self deleteSelectionFromWidget: self acrossListWidget!
   
clickedMoveRight
    self moveSelectionFromWidget: self acrossListWidget to: self downListWidget!
  
clickedStop
    (MessageBox confirm: 'Inspect') ifTrue: [^self inspect].
    puzzleScaffold pleaseStop: true!
 
clickedAddLeft
    self add: (self entryWidget contents) toWidget: self acrossListWidget!
  
clickedDeleteRight
    self deleteSelectionFromWidget: self downListWidget!

clickedMoveLeft
    self moveSelectionFromWidget: self downListWidget to: self acrossListWidget!
   
clickedMake
    CursorManager execute changeFor: [
        changed
            ifTrue: [
                puzzleScaffold
                    acrossStrings: self acrossListWidget contents
                        downStrings: self downListWidget contents;
                    reset.
                changed := false]
            ifFalse: [puzzleScaffold set]].
    self updateGraphics
! !

!PuzzleMaker methods "for 'event handling support'" !
   
moveSelectionFromWidget: widget1 to: widget2
    | selection |
    changed := true.
    selection := widget1 text.
    selection isEmpty ifTrue: [^self].
    self deleteSelectionFromWidget: widget1.
    self add: selection toWidget: widget2!
 
add: entry toWidget: widget
    | list |
    changed := true.
    entry isEmpty ifTrue: [^self].
    list := widget contents.
    list isNil ifTrue: [list := OrderedCollection new].
    list add: entry.
    list := list asSortedCollection asOrderedCollection.
    widget contents: list; selection: entry; text: entry!
   
deleteSelectionFromWidget: widget
    | selection list |
    changed := true.
    selection := widget text.
    selection isEmpty ifTrue: [^self].
    list := widget contents.
    list remove: selection.
    widget contents: list; selection: nil; text: ''.! !

!PuzzleMaker methods "for 'updating'" !
   
updateGraphics
    puzzleScaffold statusWidget: self statusWidget.
    puzzleScaffold drawOn: self puzzleWidget! !

!PuzzleMaker methods "for 'widget references'" !

entryWidget
    ^(self paneNamed: 'entryWidget')!
  
statusWidget
    ^(self paneNamed: 'statusWidget')!

puzzleWidget
    ^(self paneNamed: 'puzzleWidget')!

acrossListWidget
    ^(self paneNamed: 'acrossListWidget')!

downListWidget
    ^(self paneNamed: 'downListWidget')! !
